/****************************************************************
Copyright (c) 1998 Sandro Sigala <ssigala@globalnet.it>.
All rights reserved.

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name of the author not be used in
advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

The author disclaim all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall the author be liable for any
special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
****************************************************************/

#include "config.h"

#include <qfile.h>
#include <qtstream.h>

#include "map.h"

Map::Map()
{
	data = 0;
}

Map::~Map()
{
	if (data) delete [] data;
	deleteTransporters();
}

void Map::newMap(int xwidth, int xheight)
{
	width = xwidth;
	height = xheight;
	if (data) delete [] data;
	deleteTransporters();

	password = "none";
	time_limit = 300;

	data = new Sprite[width * height];

	for (int y = 0; y < height; ++y)
		for (int x = 0; x < width; ++x)
			xy(x, y) = Sprite_Empty;
}

void Map::save(const QString& filename)
{
	QFile f(filename);
	if (!f.open(IO_WriteOnly))
		fatal("Cannot open %s for writing", (const char *)filename);

	QTextStream t(&f);
	t << width << ' ' << height << ' ' << transporters.count() << ' ' << time_limit << '\n';
	t << password << '\n';

	for (Transporter *tr = transporters.first(); tr != 0;
	     tr = transporters.next()) {
		t << tr->orig_x << ' ' << tr->orig_y << ' '
		  << tr->dest_x << ' ' << tr->dest_y << '\n';
	}

	for (int y = 0; y < height; ++y) {
		for (int x = 0; x < width; ++x)
			t << (char)(xy(x, y) + 'A');
		t << '\n';
	}

	f.close();
}

void Map::load(const QString& filename)
{
	if (data) delete [] data;
	deleteTransporters();

	QFile f(filename);
	if (!f.open(IO_ReadOnly))
		fatal("Cannot open %s", (const char *)filename);

	QTextStream t(&f);
	int num_transporters;
	t >> width >> height >> num_transporters >> time_limit;
	t >> password;

	while (num_transporters--) {
		int ox, oy, dx, dy;
		t >> ox >> oy >> dx >> dy;
		addTransporter(ox, oy, dx, dy);
	}

	data = new Sprite[width * height];

	for (int y = 0; y < height; ++y) {
		char c;
		for (int x = 0; x < width; ++x) {
			t >> c;
			xy(x, y) = (Sprite)(c - 'A');
		}
	}

	f.close();
}

void Map::addTransporter(int ox, int oy, int dx, int dy)
{
	transporters.append(new Transporter(ox, oy, dx, dy));
}

Transporter *Map::findTransporter(int x, int y)
{
	Transporter *t;
	for (t = transporters.first(); t != 0; t = transporters.next())
		if (t->orig_x == x && t->orig_y == y)
			return t;
	fatal("Invalid transporter");
	return 0;
}

void Map::deleteTransporters()
{
	Transporter *t;
	while ((t = transporters.first()) != 0)
		transporters.remove();
}
